@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997
  *  Jamie Zawinski <jwz@netscape.com>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -22,28 +22,36 @@
 #endif
 
 #include "xdaliclock.h"
-
-#ifdef SHAPE
-extern int do_shape;
-#endif
+#include "resources.h"
+#include "visual.h"
+#include "hsv.h"
 
 int no_writable_cells;
 
 
 void
-allocate_colors (dpy, visual, cmap,
-		 fg_name, bg_name, bd_name,
-		 fg_color, bg_color, bd_color)
-     Display *dpy;
-     Visual *visual;
-     Colormap cmap;
-     char *fg_name, *bg_name, *bd_name;
-     XColor *fg_color, *bg_color, *bd_color;
+#ifdef __STDC__
+allocate_colors (Screen *screen, Visual *visual, Colormap cmap,
+		 char   *fg_name,  char   *bg_name,  char   *bd_name,
+		 XColor *fg_color, XColor *bg_color, XColor *bd_color)
+#else  /* !__STDC__ */
+allocate_colors (screen, visual, cmap, fg_name, bg_name, bd_name, fg_color,
+		 bg_color, bd_color)
+	Screen *screen;
+	Visual *visual;
+	Colormap cmap;
+	char *fg_name;
+	char *bg_name;
+	char *bd_name;
+	XColor *fg_color;
+	XColor *bg_color;
+	XColor *bd_color;
+#endif /* !__STDC__ */
 {
-  Screen *screen = DefaultScreenOfDisplay (dpy);
+  Display *dpy = DisplayOfScreen(screen);
 
-  if (! fg_name) fg_name = "black";
-  if (! bg_name) bg_name = "white";
+  if (! fg_name) fg_name = "white";
+  if (! bg_name) bg_name = "black";
 
   no_writable_cells = 0;
 
@@ -53,13 +61,18 @@ allocate_colors (dpy, visual, cmap,
 
   if (do_cycle)
     {
-      int visual_class = get_visual_class (dpy, visual);
+      int vclass = visual_class (screen, visual);
       unsigned long plane_masks;
       unsigned long pixels [2];
 
-      if (visual_class == StaticGray ||
-	  visual_class == StaticColor ||
-	  visual_class == TrueColor)
+/*      if (do_overlay)
+	{
+	  no_writable_cells = 1;
+	  goto NONWRITABLE;
+	}
+      else*/ if (vclass == StaticGray ||
+	       vclass == StaticColor ||
+	       vclass == TrueColor)
 	{
 #if 0
 	  fprintf (stderr,
@@ -76,15 +89,31 @@ allocate_colors (dpy, visual, cmap,
 #endif
 	}
       else if (XAllocColorCells (dpy, cmap, False,
-				 &plane_masks, 0, pixels, 2))
+				 &plane_masks, 0, pixels,
+				 ((do_overlay || do_shape) ? 1 : 2)))
 	{
 	  fg_color->pixel = pixels [0];
-	  bg_color->pixel = pixels [1];
+
 	  XParseColor (dpy, cmap, fg_name, fg_color);
-	  XParseColor (dpy, cmap, bg_name, bg_color);
-	  fg_color->flags = bg_color->flags = DoRed | DoGreen | DoBlue;
+	  fg_color->flags = DoRed | DoGreen | DoBlue;
 	  XStoreColor (dpy, cmap, fg_color);
-	  XStoreColor (dpy, cmap, bg_color);
+
+	  if (do_overlay)
+	    {
+	      bg_color->flags = DoRed | DoGreen | DoBlue;
+	      bg_color->red = bg_color->green = bg_color->blue = 0;
+	      bg_color->pixel = overlay_transparent_pixel;
+	    }
+	  else if (do_shape)
+	    {
+	      bg_color = fg_color;
+	    }
+	  else
+	    {
+	      bg_color->pixel = pixels [1];
+	      XParseColor (dpy, cmap, bg_name, bg_color);
+	      XStoreColor (dpy, cmap, bg_color);
+	    }
 	  no_writable_cells = 0;
 	}
       else
@@ -110,28 +139,37 @@ allocate_colors (dpy, visual, cmap,
 	{
 	  fprintf (stderr, "%s: can't parse color %s; using black\n",
 		   progname, fg_name);
-	  fg_color->pixel = BlackPixelOfScreen (screen);
+	  fg_color->pixel = WhitePixelOfScreen (screen);
 	}
       else if (! XAllocColor (dpy, cmap, fg_color))
 	{
 	  fprintf (stderr,
 		   "%s: couldn't allocate color \"%s\", using black\n",
 		   progname, fg_name);
-	  fg_color->pixel = BlackPixelOfScreen (screen);
+	  fg_color->pixel = WhitePixelOfScreen (screen);
 	}
 
-      if (! XParseColor (dpy, cmap, bg_name, bg_color))
+      if (do_overlay)
+	{
+	  bg_color->red = bg_color->green = bg_color->blue = 0;
+	  bg_color->pixel = overlay_transparent_pixel;
+	}
+      else if (do_shape)
+	{
+	  bg_color = fg_color;
+	}
+      else if (! XParseColor (dpy, cmap, bg_name, bg_color))
 	{
 	  fprintf (stderr, "%s: can't parse color %s; using white\n",
 		   progname, bg_name);
-	  bg_color->pixel = WhitePixelOfScreen (screen);
+	  bg_color->pixel = BlackPixelOfScreen (screen);
 	}
       else if (! XAllocColor (dpy, cmap, bg_color))
 	{
 	  fprintf (stderr,
 		   "%s: couldn't allocate color \"%s\", using white\n",
 		   progname, bg_name);
-	  bg_color->pixel = WhitePixelOfScreen (screen);
+	  bg_color->pixel = BlackPixelOfScreen (screen);
 	}
 
       /* kludge -rv */
@@ -148,7 +186,11 @@ allocate_colors (dpy, visual, cmap,
     bd_name = fg_name;
 
   /* Set border color to something reasonable in the colormap */
-  if (! XParseColor (dpy, cmap, bd_name, bd_color))
+  if (do_overlay || do_shape)
+    {
+      bd_color->pixel = bg_color->pixel;
+    }
+  else if (! XParseColor (dpy, cmap, bd_name, bd_color))
     {
       fprintf (stderr, "%s: can't parse color %s; using white\n",
 	       progname, bd_name);
@@ -163,13 +205,22 @@ allocate_colors (dpy, visual, cmap,
 }
 
 void
-cycle_colors (dpy, cmap, fg_color, bg_color, window, fg_gc, bg_gc)
-     Display *dpy;
-     Colormap cmap;
-     XColor *fg_color, *bg_color;
-     Window window;
-     GC fg_gc, bg_gc;
+#ifdef __STDC__
+cycle_colors (Screen *screen, Colormap cmap,
+	      XColor *fg_color, XColor *bg_color,
+	      Window window, GC fg_gc, GC bg_gc)
+#else  /* !__STDC__ */
+cycle_colors (screen, cmap, fg_color, bg_color, window, fg_gc, bg_gc)
+	Screen *screen;
+	Colormap cmap;
+	XColor *fg_color;
+	XColor *bg_color;
+	Window window;
+	GC fg_gc;
+	GC bg_gc;
+#endif /* !__STDC__ */
 {
+  Display *dpy = DisplayOfScreen(screen);
   static int hue_tick;
   hsv_to_rgb (hue_tick,
 	      1.0, 1.0,
@@ -201,7 +252,11 @@ cycle_colors (dpy, cmap, fg_color, bg_color, window, fg_gc, bg_gc)
 	}
 
       tmp = *bg_color;
-      if (!XAllocColor(dpy, cmap, bg_color))
+      if (do_overlay)
+	bg_color->pixel = overlay_transparent_pixel;
+      else if (do_shape)
+	bg_color = fg_color;
+      else if (!XAllocColor(dpy, cmap, bg_color))
 	*bg_color = tmp;
       else
 	{
@@ -216,9 +271,6 @@ cycle_colors (dpy, cmap, fg_color, bg_color, window, fg_gc, bg_gc)
       XSetForeground (dpy, bg_gc, bg_color->pixel);
       XSetBackground (dpy, bg_gc, fg_color->pixel);
       XSetWindowBackground (dpy, window,
-#ifdef SHAPE
-			    do_shape ? fg_color->pixel :
-#endif
-			    bg_color->pixel);
+			    do_shape ? fg_color->pixel : bg_color->pixel);
     }
 }
