@@ -1,5 +1,6 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995 Jamie Zawinski <jwz@netscape.com>
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997
+ *  Jamie Zawinski <jwz@netscape.com>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -21,33 +22,20 @@
 #endif
 
 #include "xdaliclock.h"
+#include "resources.h"
+#include "usleep.h"
+#include "vroot.h"
 
-extern int no_writable_cells;
-
-#ifdef __STDC__
-static struct frame *image_to_frame (XImage *image);
-static void flush_segment_buffer (Display *dpy, Drawable drawable, GC draw_gc);
-static void set_current_scanlines (struct frame *into_frame,
-				   struct frame *from_frame);
-static void draw_frame (Display *dpy, struct frame *frame, Drawable drawable,
-			GC draw_gc, int x_off);
-static void one_step (struct frame *current_frame, struct frame *target_frame,
-		      int tick);
-static void load_font (Display *dpy, char *fn);
-static void load_builtin_font (Display *dpy, Visual *visual, int which);
-static void get_resources (Display *dpy);
-static void initialize_digits (Display *dpy, Visual *visual);
-static void initialize_window (Display *dpy, Window window);
-static void fill_pig_cache (Display *dpy, Drawable drawable,
-			    struct frame *work_frame);
-static long fill_time_digits (void);
-static void wander (void);
-
-static void fill_borders (Display *dpy, Window window, GC gc,
-			  int x, int y, int width, int height);
+#define clock_usleep screenhack_usleep
 
+#ifdef SHAPE
+# include <X11/extensions/shape.h>
 #endif
 
+static void fill_borders P((Display *dpy, Window window, GC gc,
+			    int x, int y, int width, int height));
+static void draw_colon P((Display *dpy, Drawable pixmap, Window window));
+
 #if defined (BUILTIN_FONT) || defined (BUILTIN_FONT_2)
 
 static int use_builtin_font;
@@ -132,11 +120,6 @@ static int be_a_pig;
 static int minutes_only;
 static enum { MMDDYY, DDMMYY, YYMMDD, YYDDMM, MMYYDD, DDYYMM } date_format;
 
-#ifdef SHAPE
-int do_shape;
-#endif
-
-
 static Pixmap pixmap;
 static GC pixmap_draw_gc, pixmap_erase_gc;
 static GC window_draw_gc, window_erase_gc;
@@ -174,7 +157,7 @@ typedef short unsigned int POS;
    really feel like hacking on this code enough to clean it up.
  */
 #ifndef MAX_SEGS_PER_LINE
-#define MAX_SEGS_PER_LINE 3
+#define MAX_SEGS_PER_LINE 5
 #endif
 
 struct scanline {
@@ -187,8 +170,12 @@ struct frame {
 
 
 static struct frame *
+#ifdef __STDC__
+image_to_frame (XImage *image)
+#else  /* !__STDC__ */
 image_to_frame (image)
-     XImage *image;
+	XImage *image;
+#endif /* !__STDC__ */
 {
   register int y, x;
   struct frame *frame;
@@ -257,38 +244,46 @@ image_to_frame (image)
 /* This is kind of gross.
  */
 static char *default_fonts [] = {
+  "-*-minion-bold-i-*-*-*-500-*-*-*-*-*-*",
+  "-*-new century schoolbook-bold-i-*-*-*-500-*-*-*-*-*-*",
   "-*-charter-bold-i-*-*-*-500-*-*-*-*-*-*",
   "-*-lucidabright-demibold-i-*-*-*-500-*-*-*-*-*-*",
-  "-*-new century schoolbook-bold-i-*-*-*-500-*-*-*-*-*-*",
   "-*-helvetica-bold-o-*-*-*-500-*-*-*-*-*-*",
   "-*-lucida-bold-i-*-*-*-500-*-*-*-*-*-*",
+  "-*-adobe garamond-bold-i-*-*-*-500-*-*-*-*-*-*",
+  "-*-palatino-bold-i-*-*-*-500-*-*-*-*-*-*",
+  "-*-times-bold-i-*-*-*-500-*-*-*-*-*-*",
 
   "-*-charter-bold-i-*-*-*-360-*-*-*-*-*-*",
-  "-*-lucidabright-demibold-i-*-*-*-360-*-*-*-*-*-*",
   "-*-new century schoolbook-bold-i-*-*-*-360-*-*-*-*-*-*",
+  "-*-lucidabright-demibold-i-*-*-*-360-*-*-*-*-*-*",
   "-*-helvetica-bold-o-*-*-*-360-*-*-*-*-*-*",
   "-*-lucida-bold-i-*-*-*-360-*-*-*-*-*-*",
 
   "-*-charter-bold-i-*-*-*-240-*-*-*-*-*-*",
-  "-*-lucidabright-demibold-i-*-*-*-240-*-*-*-*-*-*",
   "-*-new century schoolbook-bold-i-*-*-*-240-*-*-*-*-*-*",
+  "-*-lucidabright-demibold-i-*-*-*-240-*-*-*-*-*-*",
   "-*-helvetica-bold-o-*-*-*-240-*-*-*-*-*-*",
   "-*-lucida-bold-i-*-*-*-240-*-*-*-*-*-*",	/* too wide.  bug? */
 
   "-*-charter-bold-i-*-*-*-180-*-*-*-*-*-*",
-  "-*-lucidabright-demibold-i-*-*-*-180-*-*-*-*-*-*",
   "-*-new century schoolbook-bold-i-*-*-*-180-*-*-*-*-*-*",
+  "-*-lucidabright-demibold-i-*-*-*-180-*-*-*-*-*-*",
   "-*-helvetica-bold-o-*-*-*-180-*-*-*-*-*-*",	/* too wide */
   "-*-lucida-bold-i-*-*-*-180-*-*-*-*-*-*",	/* too wide */
   0
 };
 
 static void
-load_font (dpy, fn)
-     Display *dpy;
-     char *fn;
+#ifdef __STDC__
+load_font (Screen *screen, char *fn)
+#else  /* !__STDC__ */
+load_font (screen, fn)
+	Screen *screen;
+	char *fn;
+#endif /* !__STDC__ */
 {
-  Screen *screen = DefaultScreenOfDisplay (dpy);
+  Display *dpy = DisplayOfScreen (screen);
   int i, max_lbearing, max_rbearing, max_ascent, max_descent;
   XFontStruct *font = 0;
   Pixmap pixmap;
@@ -422,11 +417,16 @@ load_font (dpy, fn)
 
 #if defined (BUILTIN_FONT) || defined (BUILTIN_FONT_2)
 static void
-load_builtin_font (dpy, visual, which)
-     Display *dpy;
-     Visual *visual;
-     int which;
+#ifdef __STDC__
+load_builtin_font (Screen *screen, Visual *visual, int which)
+#else  /* !__STDC__ */
+load_builtin_font (screen, visual, which)
+	Screen *screen;
+	Visual *visual;
+	int which;
+#endif /* !__STDC__ */
 {
+  Display *dpy = DisplayOfScreen (screen);
   int i;
 
 #if defined (BUILTIN_FONT) && defined (BUILTIN_FONT_2)
@@ -465,10 +465,10 @@ load_builtin_font (dpy, visual, which)
   XDestroyImage (image);
 
   colon_char_width = MAX (nums [10].width, nums [11].width);
-  colon = XCreateBitmapFromData (dpy, DefaultRootWindow (dpy),
+  colon = XCreateBitmapFromData (dpy, RootWindowOfScreen (screen),
 				 (char *) nums[10].bits,
 				 nums[10].width, nums[10].height);
-  slash = XCreateBitmapFromData (dpy, DefaultRootWindow (dpy),
+  slash = XCreateBitmapFromData (dpy, RootWindowOfScreen (screen),
 				 (char *) nums[11].bits,
 				 nums[11].width, nums[11].height);
 }
@@ -482,10 +482,14 @@ static XSegment segment_buffer [MAX_SEGS + 2];
 static int segment_count = 0;
 
 static void
+#ifdef __STDC__
+flush_segment_buffer (Display *dpy, Drawable drawable, GC draw_gc)
+#else  /* !__STDC__ */
 flush_segment_buffer (dpy, drawable, draw_gc)
-     Display *dpy;
-     Drawable drawable;
-     GC draw_gc;
+	Display *dpy;
+	Drawable drawable;
+	GC draw_gc;
+#endif /* !__STDC__ */
 {
   if (! segment_count) return;
   XDrawSegments (dpy, drawable, draw_gc, segment_buffer, segment_count);
@@ -495,12 +499,17 @@ flush_segment_buffer (dpy, drawable, draw_gc)
 
 
 static void
+#ifdef __STDC__
+draw_frame (Display *dpy, struct frame *frame, Drawable drawable, 
+	    GC draw_gc, int x_off)
+#else  /* !__STDC__ */
 draw_frame (dpy, frame, drawable, draw_gc, x_off)
-     Display *dpy;
-     struct frame *frame;
-     Drawable drawable;
-     GC draw_gc;
-     int x_off;
+	Display *dpy;
+	struct frame *frame;
+	Drawable drawable;
+	GC draw_gc;
+	int x_off;
+#endif /* !__STDC__ */
 {
   register int y, x;
   for (y = 0; y < character_height; y++)
@@ -525,8 +534,13 @@ draw_frame (dpy, frame, drawable, draw_gc, x_off)
 }
 
 static void
+#ifdef __STDC__
+set_current_scanlines (struct frame *into_frame, struct frame *from_frame)
+#else  /* !__STDC__ */
 set_current_scanlines (into_frame, from_frame)
-     struct frame *into_frame, *from_frame;
+	struct frame *into_frame;
+	struct frame *from_frame;
+#endif /* !__STDC__ */
 {
   register int i;
   for (i = 0; i < character_height; i++)
@@ -534,9 +548,14 @@ set_current_scanlines (into_frame, from_frame)
 }
 
 static void
+#ifdef __STDC__
+one_step (struct frame *current_frame, struct frame *target_frame, int tick)
+#else  /* !__STDC__ */
 one_step (current_frame, target_frame, tick)
-     struct frame *current_frame, *target_frame;
-     int tick;
+	struct frame *current_frame;
+	struct frame *target_frame;
+	int tick;
+#endif /* !__STDC__ */
 {
   register struct scanline *line = &current_frame->scanlines [0];
   register struct scanline *target = &target_frame->scanlines [0];
@@ -555,12 +574,35 @@ one_step (current_frame, target_frame, tick)
     }
 }
 
-static char test_hack; /* gag */
+static char test_hack = 0; /* gag */
 
 /* #define DRYRUN */
 
+
+#ifdef DRYRUN
+static void DUMP(Display *dpy, Window window, int tick)
+{
+  char file[255];
+  char buf[255];
+  char *s1 = "dalidump-%d%d:%d%d:%d%d:%d.rgb";
+  char *s2 = ("xwd -id 0x%X | (xwdtopnm | ppmtogif > tmp.gif) 2>/dev/null"
+	      "; fromgif tmp.gif %s ; rm tmp.gif");
+  if (last_time_digits[3] == -1) return;
+  sprintf (file, s1,
+	   (time_digits[0] < 0 ? 0 : time_digits[0]), time_digits[1],
+	   time_digits[2], time_digits[3],
+	   time_digits[4], time_digits[5],
+	   tick);
+  sprintf (buf, s2, window, file);
+  fprintf (stderr, "%s\n", file);
+  XSync (dpy, False);
+  system (buf);
+}
+#endif
+
+
 static long
-fill_time_digits ()
+fill_time_digits P((void))
 {
 #ifdef DRYRUN
   static long clock = 799995420L;
@@ -580,6 +622,15 @@ fill_time_digits ()
     }
   else if (! button_down)
     {
+      if (countdown)
+	{
+	  int delta = countdown - clock;
+	  if (delta < 0) delta = -delta;
+	  tm->tm_sec = delta % 60;
+	  tm->tm_min = (delta / 60) % 60;
+	  tm->tm_hour = (delta / (60 * 60)) % 60;
+	  twelve_hour_time = 0;
+	}
       if (twelve_hour_time && tm->tm_hour > 12) tm->tm_hour -= 12;
       if (twelve_hour_time && tm->tm_hour == 0) tm->tm_hour = 12;
       time_digits [0] = (tm->tm_hour - (tm->tm_hour % 10)) / 10;
@@ -640,10 +691,14 @@ fill_time_digits ()
 
 
 static void
+#ifdef __STDC__
+fill_pig_cache (Display *dpy, Drawable drawable, struct frame *work_frame)
+#else  /* !__STDC__ */
 fill_pig_cache (dpy, drawable, work_frame)
-     Display *dpy;
-     Drawable drawable;
-     struct frame *work_frame;
+	Display *dpy;
+	Drawable drawable;
+	struct frame *work_frame;
+#endif /* !__STDC__ */
 {
   int i;
   /* do `[1-9]' to `0'
@@ -709,17 +764,21 @@ fill_pig_cache (dpy, drawable, work_frame)
 
 
 static void
-initialize_digits (dpy, visual)
-     Display *dpy;
-     Visual *visual;
+#ifdef __STDC__
+initialize_digits (Screen *screen, Visual *visual)
+#else  /* !__STDC__ */
+initialize_digits (screen, visual)
+	Screen *screen;
+	Visual *visual;
+#endif /* !__STDC__ */
 {
   int i, x;
 #if defined (BUILTIN_FONT) || defined (BUILTIN_FONT_2)
   if (use_builtin_font)
-    load_builtin_font (dpy, visual, use_builtin_font);
+    load_builtin_font (screen, visual, use_builtin_font);
   else
 #endif
-    load_font (dpy, font_name);
+    load_font (screen, font_name);
 
   memset ((char *) memory_pig_zeros,  0, sizeof (memory_pig_zeros));
   memset ((char *) memory_pig_digits, 0, sizeof (memory_pig_digits));
@@ -750,42 +809,42 @@ initialize_digits (dpy, visual)
 }
 
 
-static void draw_colon P((Display *dpy, Drawable pixmap, Window window));
-
 static void
-initialize_window (dpy, window)
-     Display *dpy;
-     Window window;
+#ifdef __STDC__
+initialize_window (Screen *screen, Window window)
+#else  /* !__STDC__ */
+initialize_window (screen, window)
+	Screen *screen;
+	Window window;
+#endif /* !__STDC__ */
 {
+  Display *dpy = DisplayOfScreen (screen);
   XWindowAttributes xgwa;
-  Screen *screen = DefaultScreenOfDisplay (dpy);
   XSetWindowAttributes attributes;
   unsigned long attribute_mask;
   XGCValues gcvalues;
   int ndigits = minutes_only ? 4 : 6;
+  int have_shape = 0;
 
   XGetWindowAttributes (dpy, window, &xgwa);
 #ifdef SHAPE
-  if (!wander_p)
-    {
-      int shape_event_base, shape_error_base;
-      if (do_shape && !XShapeQueryExtension (dpy,
-					     &shape_event_base,
-					     &shape_error_base))
-	{
-	  fprintf (stderr, "%s: no shape extension on this display\n",
-		   progname);
-	  do_shape = 0;
-	}
-    }
+  {
+    int shape_event_base, shape_error_base;
+    if ((do_shape || do_overlay) &&
+	XShapeQueryExtension (dpy, &shape_event_base, &shape_error_base))
+      have_shape = 1;
+    else if (do_shape)
+      {
+	fprintf (stderr, "%s: no shape extension on this display\n", progname);
+	do_shape = 0;
+      }
+  }
 #endif
 
   attribute_mask = (CWBackPixel | CWEventMask | CWDontPropagate);
-#ifdef SHAPE
   if (do_shape)
     attributes.background_pixel = fg_color.pixel;
   else
-#endif
     attributes.background_pixel = bg_color.pixel;
   attributes.event_mask = (xgwa.your_event_mask |
 			   KeyPressMask | ButtonPressMask |
@@ -794,7 +853,7 @@ initialize_window (dpy, window)
   attributes.do_not_propagate_mask = 0;
 
   if (window != xgwa.root &&
-      window != RootWindowOfScreen (DefaultScreenOfDisplay (dpy)))
+      window != RootWindowOfScreen (screen))
     /* On SGIs, you get BadAccess if you try to frob the root window,
        or a window owned by another client.  Other servers allow this,
        however... */
@@ -803,12 +862,9 @@ initialize_window (dpy, window)
   /* Center stuff in window correctly when first created */
   {
     int width = x_offsets [minutes_only ? 3 : 5] + character_width;
-#ifdef SHAPE
-    if (do_shape)
+    if (do_shape || do_overlay)
       window_offset_x = window_offset_y = 0;
-    else
-#endif
-    if (wander_p)
+    else if (wander_p)
       {
 	window_offset_x = (WidthOfScreen (screen) - width) / 2;
 	window_offset_y = (HeightOfScreen (screen) - character_height) / 2;
@@ -853,6 +909,24 @@ initialize_window (dpy, window)
 			       GCGraphicsExposures,
 			       &gcvalues);
 
+#ifdef SHAPE
+  if (do_overlay)
+    {
+      /* If we're doing overlays, set up a rectangular shape mask anyway --
+	 this informs the window manager (mwm and twm, at least) to not put
+	 a border around the window, but to just give it a titlebar.  Since
+	 we only update this shape mask when the window is resized, this
+	 isn't a big performance problem like updating it 10x/second is.
+       */
+      XFillRectangle (dpy, pixmap, pixmap_draw_gc,
+		      0, 0, x_offsets [ndigits-1] + character_width + 1,
+		      character_height + 1);
+      XShapeCombineMask (dpy, window, ShapeBounding,
+			 window_offset_x, window_offset_y, pixmap, ShapeSet);
+    }
+#endif      
+
+
   XFillRectangle (dpy, pixmap, pixmap_erase_gc,
 		  0, 0, x_offsets [ndigits-1] + character_width + 1,
 		  character_height + 1);
@@ -870,8 +944,12 @@ initialize_window (dpy, window)
 
 
 static void
-get_resources (dpy)
-     Display *dpy;
+#ifdef __STDC__
+get_resources (Screen *screen)
+#else  /* !__STDC__ */
+get_resources (screen)
+	Screen *screen;
+#endif /* !__STDC__ */
 {
   char *buf;
 
@@ -885,12 +963,6 @@ get_resources (dpy)
   if (!strcasecmp (font_name, "BUILTIN")) use_builtin_font = 1;
   if (!strcasecmp (font_name, "BUILTIN2")) use_builtin_font = 2;
 #endif /* BUILTIN_FONT || BUILTIN_FONT_2 */
-#ifdef SHAPE
-  if (wander_p)
-    do_shape = 0;
-  else
-    do_shape = get_boolean_resource ("shape", "Shape");
-#endif
 
   buf = get_string_resource ("mode", "Mode");
   if (buf == 0 || !strcmp (buf, "12"))
@@ -961,7 +1033,7 @@ get_resources (dpy)
 
 
 static void
-wander ()
+wander P((void))
 {
   wander_x += wander_delta_x;
   if (wander_x == character_width || wander_x == 0)
@@ -975,10 +1047,14 @@ wander ()
 
 
 static void
+#ifdef __STDC__
+draw_colon (Display *dpy, Drawable pixmap, Window window)
+#else  /* !__STDC__ */
 draw_colon (dpy, pixmap, window)
-     Display *dpy;
-     Drawable pixmap;
-     Window window;
+	Display *dpy;
+	Drawable pixmap;
+	Window window;
+#endif /* !__STDC__ */
 {
   Pixmap glyph = button_down ? slash : colon;
   XCopyPlane (dpy, glyph, pixmap, pixmap_draw_gc,
@@ -1002,22 +1078,30 @@ draw_colon (dpy, pixmap, window)
  */
 
 
-static int event_loop P((Display *, Window));
+static int event_loop P((Screen *, Window));
 
 void
-initialize_digital (dpy, visual, cmap,
-		    fgP, bgP, bdP, widthP, heightP)
-     Display *dpy;
-     Visual *visual;
-     Colormap cmap;
-     unsigned long *fgP, *bgP, *bdP;
-     unsigned int *widthP, *heightP;
+#ifdef __STDC__
+initialize_digital (Screen *screen, Visual *visual, Colormap cmap,
+		    unsigned long *fgP, unsigned long *bgP, unsigned long *bdP,
+		    unsigned int *widthP, unsigned int *heightP)
+#else  /* !__STDC__ */
+initialize_digital (screen, visual, cmap, fgP, bgP, bdP, widthP, heightP)
+	Screen *screen;
+	Visual *visual;
+	Colormap cmap;
+	unsigned long *fgP;
+	unsigned long *bgP;
+	unsigned long *bdP;
+	unsigned int *widthP;
+	unsigned int *heightP;
+#endif /* !__STDC__ */
 {
   int ndigits;
-  get_resources (dpy);
-  initialize_digits (dpy, visual);
+  get_resources (screen);
+  initialize_digits (screen, visual);
 
-  allocate_colors (dpy, visual, cmap,
+  allocate_colors (screen, visual, cmap,
 		   get_string_resource ("foreground", "Foreground"),
 		   get_string_resource ("background", "Background"),
 		   get_string_resource ("borderColor", "Foreground"),
@@ -1033,15 +1117,20 @@ initialize_digital (dpy, visual, cmap,
 
 
 void
-run_digital (dpy, window)
-     Display *dpy;
-     Window window;
+#ifdef __STDC__
+run_digital (Screen *screen, Window window)
+#else  /* !__STDC__ */
+run_digital (screen, window)
+	Screen *screen;
+	Window window;
+#endif /* !__STDC__ */
 {
+  Display *dpy = DisplayOfScreen (screen);
   int i;
   int ndigits = (minutes_only ? 4 : 6);
   XWindowAttributes xgwa;
   XGetWindowAttributes (dpy, window, &xgwa);
-  initialize_window (dpy, window);
+  initialize_window (screen, window);
 
   if (be_a_pig)
     fill_pig_cache (dpy, window, current_frames [0]);
@@ -1052,7 +1141,7 @@ run_digital (dpy, window)
       set_current_scanlines (current_frames [i], clear_frame);
     }
 
-  event_loop (dpy, window); /* wait for initial mapwindow event */
+  event_loop (screen, window); /* wait for initial mapwindow event */
 
   XFillRectangle (dpy, window, window_erase_gc,
 		  0, 0, x_offsets [ndigits-1] + character_width,
@@ -1091,7 +1180,7 @@ run_digital (dpy, window)
 	{
 	  int j;
 	  if (do_cycle)
-	    cycle_colors (dpy, xgwa.colormap, &fg_color, &bg_color,
+	    cycle_colors (screen, xgwa.colormap, &fg_color, &bg_color,
 			  window, window_draw_gc, window_erase_gc);
 	  for (j = 0; j < ndigits; j++)
 	    {
@@ -1180,7 +1269,8 @@ run_digital (dpy, window)
 #ifdef SHAPE
 	  if (do_shape)
 	    XShapeCombineMask (dpy, window, ShapeBounding,
-			       0, 0, pixmap, ShapeSet);
+			       window_offset_x, window_offset_y,
+			       pixmap, ShapeSet);
 	  else
 #endif
 	    /* sends 28 bytes */
@@ -1191,11 +1281,7 @@ run_digital (dpy, window)
 			window_offset_y + (wander_y - (character_height / 2)),
 			1);
 
-	  if (do_cycle &&
-#ifdef SHAPE
-	      !do_shape &&
-#endif
-	      no_writable_cells)
+	  if (do_cycle && no_writable_cells)
 	    fill_borders (dpy, window, window_erase_gc,
 			  window_offset_x + (wander_x - (character_width / 2)),
 			  window_offset_y + (wander_y - (character_height /2)),
@@ -1204,7 +1290,7 @@ run_digital (dpy, window)
 
 	  XFlush (dpy);
 #ifdef DRYRUN
-	  getc(stdin);
+	  DUMP(dpy, window, 9-tick);
 #else
 	  TICK_SLEEP ();
 #endif
@@ -1216,16 +1302,13 @@ run_digital (dpy, window)
 
 #ifdef SHAPE
       if (do_shape)
-	XShapeCombineMask (dpy, window, ShapeBounding, 0, 0,
+	XShapeCombineMask (dpy, window, ShapeBounding,
+			   window_offset_x, window_offset_y,
 			   pixmap, ShapeSet);
       else
 #endif
       {
-	if (wander_p && different_minute
-#ifdef SHAPE
-	    && !do_shape
-#endif
-	    )
+	if (wander_p && different_minute)
 	  {
 #if 0
 	    /* XClearWindow() depends on us being able to set the background
@@ -1247,11 +1330,7 @@ run_digital (dpy, window)
 		  window_offset_y + (wander_y - (character_height / 2)),
 		  1);
 
-      if (do_cycle &&
-#ifdef SHAPE
-	  !do_shape &&
-#endif
-	  no_writable_cells)
+      if (do_cycle && no_writable_cells)
 	fill_borders (dpy, window, window_erase_gc,
 		      window_offset_x + (wander_x - (character_width / 2)),
 		      window_offset_y + (wander_y - (character_height / 2)),
@@ -1259,8 +1338,7 @@ run_digital (dpy, window)
 		      character_height);
 
 #ifdef DRYRUN
-      event_loop (dpy, window);
-      getc(stdin);
+      event_loop (screen, window);
 #else
     if (minutes_only)
       {
@@ -1278,7 +1356,7 @@ run_digital (dpy, window)
 	    /* if event_loop returns true, we need to go and repaint stuff
 	       right now, instead of waiting for the minute to elapse.
 	       */
-	    if (event_loop (dpy, window))
+	    if (event_loop (screen, window))
 	      break;
 
 	    if (now == target-1)	/* the home stretch; sync up */
@@ -1286,7 +1364,7 @@ run_digital (dpy, window)
 	    else
 	      {
 		if (do_cycle)
-		  cycle_colors (dpy, xgwa.colormap, &fg_color, &bg_color,
+		  cycle_colors (screen, xgwa.colormap, &fg_color, &bg_color,
 				window, window_draw_gc, window_erase_gc);
 		SEC_SLEEP ();
 	      }
@@ -1306,7 +1384,11 @@ run_digital (dpy, window)
 	  {
 	    TICK_SLEEP ();
 	    now = time ((time_t *) 0);
-	    event_loop (dpy, window);
+	    /* if event_loop returns true, we need to go and repaint stuff
+	       right now, instead of waiting for the second to elapse.
+	     */
+	    if (event_loop (screen, window))
+	      break;
 	  }
       }
 #endif
@@ -1315,11 +1397,19 @@ run_digital (dpy, window)
 
 
 static void
+#ifdef __STDC__
+fill_borders (Display *dpy, Window window, GC gc,
+	      int x, int y, int width, int height)
+#else  /* !__STDC__ */
 fill_borders (dpy, window, gc, x, y, width, height)
-     Display *dpy;
-     Window window;
-     GC gc;
-     int x, y, width, height;
+	Display *dpy;
+	Window window;
+	GC gc;
+	int x;
+	int y;
+	int width;
+	int height;
+#endif /* !__STDC__ */
 {
   XRectangle rects[4];
   int i = 0;
@@ -1354,10 +1444,15 @@ fill_borders (dpy, window, gc, x, y, width, height)
 }
 
 static int
-event_loop (dpy, window)
-     Display *dpy;
-     Window window;
+#ifdef __STDC__
+event_loop (Screen *screen, Window window)
+#else  /* !__STDC__ */
+event_loop (screen, window)
+	Screen *screen;
+	Window window;
+#endif /* !__STDC__ */
 {
+  Display *dpy = DisplayOfScreen (screen);
   static int mapped_p = 0;
   int wait_for_buttonup = button_down;
   int redraw_p = 0;
@@ -1421,11 +1516,34 @@ event_loop (dpy, window)
 	case ConfigureNotify:
 	  {
 	    int width = x_offsets [minutes_only ? 3 : 5] + character_width;
-#ifdef SHAPE
-	    if (do_shape) break;
-#endif
+
 	    window_offset_x = (event.xconfigure.width - width) / 2;
 	    window_offset_y = (event.xconfigure.height - character_height) / 2;
+
+#ifdef SHAPE
+	    if (do_overlay)
+	      {
+		/* If we're doing overlays, set up a rectangular shape mask
+		   anyway -- this informs the window manager (mwm and twm, at
+		   least) to not put a border around the window, but to just
+		   give it a titlebar.  Since we only update this shape mask
+		   when the window is resized, this isn't a big performance
+		   problem like updating it 10x/second is.
+		 */
+		int ndigits = (minutes_only ? 4 : 6);
+		int w = x_offsets [ndigits-1] + character_width;
+		int h = character_height + 1;
+		Pixmap p =
+		  XCreatePixmap (dpy, RootWindowOfScreen (screen), w, h, 1);
+		XFillRectangle (dpy, p, pixmap_draw_gc, 0, 0, w, h);
+		XShapeCombineMask (dpy, window, ShapeBounding,
+				   window_offset_x, window_offset_y,
+				   p, ShapeSet);
+		XFreePixmap (dpy, p);
+	      }
+#endif
+
+
 #if 0
 	    /* XClearWindow() depends on us being able to set the background
 	       color of the window, which we can't necessarily do if it is the
@@ -1450,4 +1568,3 @@ event_loop (dpy, window)
     }
   return redraw_p;
 }
-
