@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997
  *  Jamie Zawinski <jwz@netscape.com>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -20,9 +20,8 @@
  * algorithm ran perfectly well on a Macintosh with a 128k address space.)
  */
 
-#include "version.h"
-
 #include <stdio.h>
+#include <ctype.h>
 #include <X11/Intrinsic.h>
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
@@ -30,15 +29,30 @@
 #include <X11/CoreP.h>
 
 #include "xdaliclock.h"
-#include "defaults.h"
+#include "vroot.h"
+#include "version.h"
+#include "resources.h"
+#include "visual.h"
 
 char *progname;
 char *progclass;
-char *hacked_version;
+char *hacked_version, *hacked_version2;
 XrmDatabase db;
 Bool wander_p;
 Bool root_p;
 Bool do_cycle;
+long countdown;
+
+int do_shape = 0;
+int do_overlay = 0;
+unsigned long overlay_transparent_pixel = 0;
+
+
+static char *defaults[] = {
+#include "XDaliClock_ad.h"
+ 0
+};
+
 
 static void usage P((void));
 static void hack_version P((void));
@@ -73,20 +87,26 @@ static XrmOptionDescRec options [] = {
   { "-sleaze-level",	".memory",		XrmoptionSepArg, 0 },
   { "-oink",		".memory",		XrmoptionNoArg, "Medium" },
   { "-oink-oink",	".memory",		XrmoptionNoArg, "High" },
-  { "-shape",		".shape",		XrmoptionNoArg, "True" },
-  { "-noshape",		".shape",		XrmoptionNoArg, "False" }
+  { "-transparent",	".transparent",		XrmoptionNoArg, "True" },
+  { "-nontransparent",	".transparent",		XrmoptionNoArg, "False" },
+  { "-countdown",	".countdown",		XrmoptionSepArg, 0 }
 };
 
 int
+#ifdef __STDC__
+main (int argc, char **argv)
+#else  /* !__STDC__ */
 main (argc, argv)
-     int argc;
-     char **argv;
+	int argc;
+	char **argv;
+#endif /* !__STDC__ */
 {
   Widget toplevel = 0;
   Display *dpy;
+  Screen *screen;
   unsigned int w, h;
-  Window window;
-  Visual *visual;
+  Window window = 0;
+  Visual *visual = 0;
   Colormap cmap;
   unsigned long fg, bg, bd;
   XWindowAttributes xgwa;
@@ -110,6 +130,7 @@ main (argc, argv)
     dpy = XtOpenDisplay (app, 0, 0, progclass,
 			 options, sizeof (options) / sizeof (options [0]),
 			 &argc, argv);
+    if (!dpy) exit (-1);
     XtGetApplicationNameAndClass (dpy, &progname, &progclass);
     db = XtDatabase (dpy);
   }
@@ -157,41 +178,71 @@ main (argc, argv)
   }
 #endif /* R3 */
 
+  if (toplevel)
+    screen = XtScreen (toplevel);
+  else
+    screen = DefaultScreenOfDisplay(dpy);
+
   if (argc > 1)
     {
-      if (strcmp (argv [1], "-help"))
+      int help_p = !strcmp (argv [1], "-help");
+      if (!help_p)
 	fprintf (stderr, "%s: unrecognised option \"%s\"\n",
 		 progname, argv[1]);
       usage ();
-      exit (-1);
+      exit (help_p ? 0 : -1);
     }
 
   root_p = get_boolean_resource ("root", "Boolean");
   full_screen_p = get_boolean_resource ("fullScreen", "Boolean");
   do_cycle = get_boolean_resource ("cycle", "Cycle");
 
+  countdown = 0;
+  {
+    char *s = get_string_resource ("countdown", "Countdown");
+    if (s && *s)
+      {
+	/* #### write date parser here (for the thousandth time) */
+	countdown = atol(s);
+      }
+    if (s) free(s);
+  }
+
   if (root_p)
     {
-      window = RootWindowOfScreen (DefaultScreenOfDisplay (dpy));
+      window = RootWindowOfScreen (screen);
       XGetWindowAttributes (dpy, window, &xgwa);
       cmap = xgwa.colormap;
       visual = xgwa.visual;
     }
   else
     {
-      /* Note: we can't use the resource ".visual" because Xt is SO FUCKED. */
-      visual = get_visual_resource (dpy, "visualID", "VisualID");
-      if ((visual != DefaultVisual (dpy, DefaultScreen (dpy))) ||
+      do_shape = get_boolean_resource ("transparent", "Transparent");
+      if (do_shape)
+	{
+	  Visual *v = get_overlay_visual (screen, &overlay_transparent_pixel);
+	  if (v)
+	    {
+	      do_shape = 0;
+	      do_overlay = 1;
+	      visual = v;
+	    }
+	}
+
+      if (!do_overlay)
+	visual = get_visual_resource (screen, "visualID", "VisualID", True);
+
+      if ((visual != DefaultVisualOfScreen (screen)) ||
 	  get_boolean_resource ("installColormap", "InstallColormap"))
-	cmap = XCreateColormap (dpy, RootWindow (dpy, DefaultScreen (dpy)),
+	cmap = XCreateColormap (dpy, RootWindowOfScreen (screen),
 				visual, AllocNone);
       else
-	cmap = DefaultColormap (dpy, DefaultScreen (dpy));
+	cmap = DefaultColormapOfScreen (screen);
     }
 
   wander_p = (root_p || full_screen_p);
 
-  initialize_digital (dpy, visual, cmap, &fg, &bg, &bd, &w, &h);
+  initialize_digital (screen, visual, cmap, &fg, &bg, &bd, &w, &h);
 
   if (root_p)
     {
@@ -211,7 +262,7 @@ main (argc, argv)
       XtSetArg (av [ac], XtNinput, True); ac++;
       XtSetArg (av [ac], XtNvisual, visual); ac++;
       XtSetArg (av [ac], XtNcolormap, cmap); ac++;
-      XtSetArg (av [ac], XtNdepth, get_visual_depth (dpy, visual)); ac++;
+      XtSetArg (av [ac], XtNdepth, visual_depth (screen, visual)); ac++;
       XtSetArg (av [ac], XtNforeground, ((Pixel) fg)); ac++;
       XtSetArg (av [ac], XtNbackground, ((Pixel) bg)); ac++;
       XtSetArg (av [ac], XtNborderColor, (Pixel) bd); ac++;
@@ -236,8 +287,8 @@ main (argc, argv)
 
       if (full_screen_p)
 	{
-	  w = DisplayWidth  (dpy, DefaultScreen (dpy));
-	  h = DisplayHeight (dpy, DefaultScreen (dpy));
+	  w = WidthOfScreen  (screen);
+	  h = HeightOfScreen (screen);
 	}
       if (ow <= 1 || oh <= 1)
 	{
@@ -264,113 +315,110 @@ main (argc, argv)
 #if (XtSpecificationRelease >= 4)
   if (!root_p)
     XSetStandardProperties(dpy, window, progname, progname,
-			   NULL, /* icon pixmap */
+			   0, /* icon pixmap */
 			   argv_copy, argc_copy,
 			   NULL /* XSizeHints* */);
   free(argv_copy);
 #endif
 
-  run_digital (dpy, window);
+  run_digital (screen, window);
   return (0);
 }
 
 
 static void
-usage ()
+usage P((void))
 {
-  fprintf (stderr, "%s\n\
+  fprintf (stderr, "\n%s\n\
+                  http://people.netscape.com/jwz/xdaliclock/\n\n\
 usage: %s [ options ]\n\
-where options are\n\
+where options include\n\
 \n\
-  -12				Display twelve hour time (default).\n\
-  -24				Display twenty-four hour time.\n\
-  -seconds			Display seconds (default).\n\
-  -noseconds			Don't display seconds.\n\
-  -cycle			Do color-cycling.\n\
-  -nocycle			Don't do color-cycling (default).\n\
-  -display <host:dpy>		The display to run on.\n\
-  -visual <visual-class>	The visual to use.\n\
-  -geometry <geometry>		Size and position of window.\n\
-  -foreground or -fg <color>	Foreground color (default black).\n\
-  -background or -bg <color>	Background color (default white).\n\
-  -reverse or -rv		Swap foreground and background.\n\
-  -borderwidth or -bw <int>	Border width.\n\
-  -bd or -border <color>	Border color.\n\
-  -title <string>		Window title.\n\
-  -name <string>		Resource-manager class name (XDaliClock).\n\
-  -fullscreen			Use a window that takes up the whole screen.\n\
-  -root				Draw on the root window instead.\n\
-", version + 4, progname);
+  -12                           Display twelve hour time (default).\n\
+  -24                           Display twenty-four hour time.\n\
+  -seconds                      Display seconds (default).\n\
+  -noseconds                    Don't display seconds.\n\
+  -cycle                        Do color-cycling.\n\
+  -nocycle                      Don't do color-cycling (default).\n\
+  -display <host:dpy>           The display to run on.\n\
+  -visual <visual-class>        The visual to use.\n\
+  -install                      Allocate a private colormap.\n\
+  -geometry <geometry>          Size and position of window.\n\
+  -foreground or -fg <color>    Foreground color (default white).\n\
+  -background or -bg <color>    Background color (default black).\n\
+  -reverse or -rv               Swap foreground and background.\n\
+  -borderwidth or -bw <int>     Border width.\n\
+  -bd or -border <color>        Border color.\n\
+  -title <string>               Window title.\n\
+  -name <string>                Resource-manager name (%s).\n\
+  -fullscreen                   Use a window that takes up the whole screen.\n\
+  -root                         Draw on the root window instead.\n\
+  -window                       Opposite of -root (default).\n\
+  -datemode <MMDDYY | DDMMYY>   How to format the date.\n\
+", hacked_version2, progname, progname);
 #if defined (BUILTIN_FONT) && defined (BUILTIN_FONT_2)
   fprintf (stderr, "\
-  -font	or -fn <font>		Name of an X font to use, or the string\n\
-				\"BUILTIN\", meaning to use the large builtin\n\
-				font (this is the default), or \"BUILTIN2\",\n\
-				meaning to use the even larger builtin font.\n\
-  -builtin			Same as -font BUILTIN.\n\
-  -builtin2			Same as -font BUILTIN2.\n\
+  -font or -fn <font>           Name of an X font to use, or the string\n\
+                                \"BUILTIN\", meaning to use the large builtin\n\
+                                font (this is the default), or \"BUILTIN2\",\n\
+                                meaning to use the even larger builtin font.\n\
+  -builtin                      Same as -font BUILTIN.\n\
+  -builtin2                     Same as -font BUILTIN2.\n\
 ");
 #else /* ! (BUILTIN_FONT && BUILTIN_FONT_2) */
 #if (defined (BUILTIN_FONT) || defined (BUILTIN_FONT_2))
   fprintf (stderr, "\
-  -font	or -fn <font>		Name of an X font to use, or the string\n\
-				\"BUILTIN\", meaning to use the large builtin\n\
-				font (this is the default).\n\
-  -builtin			Same as -font BUILTIN.\n\
+  -font or -fn <font>           Name of an X font to use, or the string\n\
+                                \"BUILTIN\", meaning to use the large builtin\n\
+                                font (this is the default).\n\
+  -builtin                      Same as -font BUILTIN.\n\
 ");
 #else /* ! (BUILTIN_FONT || BUILTIN_FONT_2) */
   fprintf (stderr, "\
-  -font	or -fn <font>		Name of an X font to use.\n\
+  -font or -fn <font>           Name of an X font to use.\n\
 ");
 #endif /* ! (BUILTIN_FONT || BUILTIN_FONT_2) */
 #endif /* ! (BUILTIN_FONT && BUILTIN_FONT_2) */
   fprintf (stderr, "\
-  -memory <high|medium|low>	Tune the memory versus bandwidth consumption;\n\
-				default is \"low\".\n\
-  -oink				Same as -memory medium.\n\
-  -oink-oink			Same as -memory high.\n\
-");
-#ifdef SHAPE
-  fprintf (stderr, "\
-  -shape			Use the Shape extension if available.\n\
-  -noshape			Don't.\n");
+  -memory <high|medium|low>     Tune the memory versus bandwidth consumption;\n\
+                                default is \"low\".\n\
+  -oink                         Same as -memory medium.\n\
+  -oink-oink                    Same as -memory high.\n\
+  -transparent                  Make the window background be transparent,\n\
+                                if possible.\n\
+  -nontransparent               Don't.\n\n");
+#if !defined(BUILTIN_FONT) && !defined(BUILTIN_FONT_2)
+  fprintf(stderr,
+	  "This version has been compiled without the builtin fonts.\n\n");
 #endif
-#if !(defined (BUILTIN_FONT) || defined (BUILTIN_FONT_2)) || !defined (SHAPE)
-  fprintf (stderr, "\n");
-# ifndef BUILTIN_FONT
-   fprintf(stderr, "This version has been compiled without the builtin fonts");
-# endif
-# ifndef SHAPE
-#  ifndef BUILTIN_FONT
-  fprintf (stderr, ", or support for the\nShape Extension.\n");
-#  else
-  fprintf (stderr, "\
-This version has been compiled without support for the Shape Extension.\n");
-#  endif
-# else
-  fprintf (stderr, ".\n");
-# endif
-#endif
-  exit (-1);
 }
 
 static void
-hack_version ()
+hack_version P((void))
 {
-  char *src = version + 4;
-  char *dst = hacked_version = (char *) malloc (strlen (src) + 1);
+  int i;
+  const char *src = version + 4;
+  char *dst1 = hacked_version  = (char *) malloc (strlen (src) + 1);
+  char *dst2 = hacked_version2 = (char *) malloc (strlen (src) + 1);
 #define digitp(x) (('0' <= (x)) && ((x) <= '9'))
   while (*src)
     if (!strncmp ("Copyright (c)", src, 13))
-      *dst++ = '\251', src += 13;
-    else if (*src == '(') *dst++ = '<', src++;
-    else if (*src == ')') *dst++ = '>', src++;
+      {
+	*dst1++ = '\251';
+	for (i = 0; i < 13; i++) *dst2++ = *src++;
+      }
+    else if (*src == '(') *dst1++ = *dst2++ = '<', src++;
+    else if (*src == ')') *dst1++ = *dst2++ = '>', src++;
     else if (digitp (src[0]) && digitp (src[1]) && digitp (src[2]) &&
-	     digitp (src[3]) && src[4] == ',' && src[5] == ' ')
-      src += 6;
+	     digitp (src[3]) && src[4] == ',')
+      {
+	src += 5;
+	if (*src == ' ') src++;
+      }
     else
-      *dst++ = *src++;
-  *dst = 0;
+      *dst1++ = *dst2++ = *src++;
+  *dst1 = 0;
+  *dst2 = 0;
 }
 
 #ifdef VMS
